/********************************************************************************
 * changeowner (linkable object file)                                           *
 *                                                                              *
 *  Author: Kai Peter, ©2020-                                                   *
 * License: ISC                                                                 *
 * Version: 0.4                                                                 *
 *                                                                              *
 * Description: Changing the ownership of files requires  privilege escalation  *
 *              for ordinary users, thus it is easier to use 'sudo' in a shell  *
 *              instead of the syscall. Using 'execlp' prevents the definition  *
 *              of hardcoded pathes. According to the man page 'chown(2)', the  *
 *              syscall 'chown()' could fail on 4.4BSD.                         *
 *              Parameters: filename, user[.group] (group is optional)          *
 *              Returns 0 on success or -1 otherwise.                           *
 *******************************************************************************/
#define ME ""

#include <unistd.h>
#include <sys/stat.h>
#include <pwd.h>
#include <grp.h>
#include "qstrings.h"
#include "wait.h"

#include "changeowner.h"

int chgOwner(char *f,const char *u) {
  int pid;
  int wstat;
  stralloc sa = {0};

  if (str_len(u) == 0) u = sysusr;
  pid = fork();
  if (!pid) {
    if (!stralloc_copys(&sa,"sudo -u qemud chown ")) errmem;
    if (!stralloc_cats(&sa,u)) errmem;
    if (!stralloc_cats(&sa," ")) errmem;
    if (!stralloc_cats(&sa,f)) errmem;
    if (!stralloc_0(&sa)) errmem;
    /* The error handling is for 'execlp', NOT for errors of shell commands */
    if(execlp("sh", "sh", "-c", sa.s, (char *) NULL) < 0) return(-1);
  }
  wait_pid(&wstat,pid);

  /* verify that the uid/gid was set correctily */
  struct stat st;
  struct passwd *pwd;
  struct group *grp;

  if (stat(f,&st) != 0) errsys(errno);

  int i = instr((char *)u,".") - 1;
//out(fmtnum(i));
  if (i < 0) {    /* no group in 'u' */
//  u = "kp";
    pwd = getpwnam(u);
//pwd = getpwnam(strsplit(u,4,'L'));
//out(u);
    if (pwd->pw_uid != st.st_uid) return(-1);
    return(0);
  }
  /* 'u' has the format user.group */
  pwd = getpwnam(strsplit((char *)u,i,'L'));
  grp = getgrnam(strsplit((char *)u,i,'R'));
//out(pwd->pw_name);
//out(fmtnum(pwd->pw_uid));
//out(grp->gr_name);
//out(fmtnum(grp->gr_gid));
//out(strsplit(u,i,1));
  if (pwd->pw_uid != st.st_uid) return(-1);
  if (grp->gr_gid != st.st_gid) return(-1);

  return(0);
}
