/********************************************************************************
 * register (linkable object file)                                              *
 *                                                                              *
 *  Author: Kai Peter, ©2016-                                                   *
 * License: ISC                                                                 *
 * Version: 0.3                                                                 *
 *                                                                              *
 * Description: Register a virtual machine for usage with qemu-init.            *
 *******************************************************************************/
#define ME "register"

#include <unistd.h>
#include <sys/stat.h>
#include "buffer.h"
#include "case.h"
#include "errmsg.h"
#include "getln.h"
#include "qstrings.h"
#include "open.h"
#include "wait.h"

#include "changeowner.h"
#include "convertname.h"
#include "register.h"
#include "makepath.h"

#include "ac-homedir.c"

char buf[128];
buffer b;

/* option flags */
extern int flag_user;

ssize_t mywrite(int fd,char *buf,int len) {
  buffer_put(&b,buf,len);
  return len;
}

int doRegister(void) {
  int fd;
  char *fn = regfile;
  int pos;
  stralloc t = {0};    /* temporary variable */
  stralloc src = {0};
  stralloc dst = {0};
  struct stat st;
  int pid;
  int wstat;

  /* if no path is given, use the default location (path.s contains '\0' only */
  if (path.len == 1) {
    if (!stralloc_copys(&path,home)) errmem;
    if (!stralloc_cats(&path,"/dat")) errmem;
    if (!stralloc_0(&path)) errmem;
  }

  /* prepend 'HOMEDIR/dat' to relative path */
  if (path.s[0] != '/') {
    stralloc_init(&t);
    if (!stralloc_copys(&t,home)) errmem;
    if (!stralloc_cats(&t,"/dat/")) errmem;
    if (!stralloc_cat(&t,&path)) errmem;
    if (!stralloc_0(&t)) errmem;
    stralloc_init(&path);
    if (!stralloc_copy(&path,&t)) errmem;
  }

  /* check for trailing slash at 'path' (required with option'-p' only */
  for(;;) {
    if (path.s[str_len(path.s) - 1] != '/') break;
    pos = byte_rchr(path.s,str_len(path.s),'/');
    if (!stralloc_copys(&path,strsplit(path.s,pos,QLEFT))) errmem;
    if (!stralloc_0(&path)) errmem;
  }

  if (str_equal(name.s,path.s)) errint(EINVAL,B("Invalid name: '",name.s,"'!"));

  if (IsRegistered(&lnam) == 1)
    errint(EEXIST,B("A virtual machine called ",name.s," is registered already!"));

  /* if the option '-U' is given, change the user to 'qemud' */
  if (stat(path.s,&st) != 0) {
    log(B("Creating folder ",path.s));
	if (stat(path.s,&st) != 0) {
	  if (mkpath(path.s,0775) != 0) errsys(errno);
      if (chmod(path.s,02775) != 0) errint(errno,path.s); /* writable by group */
    }
    if (flag_user)
      if (chgOwner(path.s,sysusr) != 0)
        errint(errno,B("Unable to change owner of ",path.s,"!"));
  }

  if (chdir(path.s) == -1) errint(errno,B("Unable to switch to ",path.s,"!"));
  umask(002);

  /* create '*-qemud.rc' file from '$HOME/etc/template-qemud.rc' */
  if (!stralloc_copys(&dst,lnam.s)) errmem;
  if (!stralloc_cats(&dst,"-qemud.rc")) errmem;
  if (!stralloc_0(&dst)) errmem;
  if (stat(dst.s,&st) != 0) {
    stralloc_init(&src);
    if (!stralloc_copys(&src,home)) errmem;
    if (!stralloc_cats(&src,"/etc/template-qemud.rc")) errmem;
    if (!stralloc_0(&src)) errmem;
    stralloc_init(&t);
    if (!stralloc_copys(&t,"cat ")) errmem;
    if (!stralloc_cats(&t,src.s)) errmem;
    if (!stralloc_cats(&t," | sed \"s/[<]name[>]/")) errmem;
    if (!stralloc_cats(&t,name.s)) errmem;
    if (!stralloc_cats(&t,"\"/g")) errmem;
    if (!stralloc_cats(&t," | sed \"s/[<]name-sda[>]/")) errmem;
    if (!stralloc_cats(&t,lnam.s)) errmem;
    if (!stralloc_cats(&t,"-sda\"/g")) errmem;
    if (!stralloc_cats(&t," > ")) errmem;
    if (!stralloc_cats(&t,dst.s)) errmem;
    if (!stralloc_0(&t)) errmem;
    pid = fork();
    if (!pid)
      if (execlp("sh", "sh" , "-c", t.s, (char *) NULL) < 0)
        errint(errno,B("Couldn't create ",dst.s,"!"));  /* ??? does it work ??? */
    if(wait_pid(&wstat,pid) < 0)
      errint(errno,B("Couldn't create ",dst.s,"!"));
    /* change owner with '-U' */
    if (flag_user)
      if (chgOwner(dst.s,"") != 0)
        errint(errno,B("Couldn't create ",dst.s,"!"));
  }

  /* create the '-qemu.cfg' file from 'etc/template-qemu.cfg' */
  if (!stralloc_copys(&dst,lnam.s)) errmem;
  if (!stralloc_cats(&dst,"-qemu.cfg")) errmem;
  if (!stralloc_0(&dst)) errmem;
  if (stat(dst.s,&st) != 0) {
    stralloc_init(&src);
    if (!stralloc_copys(&src,home)) errmem;
    if (!stralloc_cats(&src,"/etc/template-qemu.cfg")) errmem;
    if (!stralloc_0(&src)) errmem;
    stralloc_init(&t);
    if (!stralloc_copys(&t,"cp ")) errmem;
    if (!stralloc_cats(&t,src.s)) errmem;
    if (!stralloc_cats(&t," ")) errmem;
    if (!stralloc_cats(&t,dst.s)) errmem;
    if (!stralloc_0(&t)) errmem;
    pid = fork();
    if (!pid)
      if (execlp("sh", "sh" , "-c", t.s, (char *) NULL) < 0)
        errint(errno,B("Couldn't create ",dst.s,"!"));
    if(wait_pid(&wstat,pid) < 0)
      errint(errno,B("Couldn't create ",dst.s,"!"));
	if (chmod(dst.s, 0664) !=0) errint(errno,t.s);
    /* change owner with '-U' */
    if (flag_user)
      if (chgOwner(dst.s,"") != 0)
        errint(errno,B("Couldn't create ",dst.s,"!"));
  }

  /* last step: write it into registered file */
  stralloc_init(&t);
  if (!stralloc_copys(&t,home)) errmem;
  if (!stralloc_cats(&t,"/etc")) errmem;
  if (!stralloc_0(&t)) errmem;
  if (chdir(t.s) == -1) errint(errno,B("Unable to switch to '",home,"/etc'!"));
  fd = open_append(fn);
  if (fd < 0) errint(errno,B("Couldn't open ",fn,"!"));
  buffer_init(&b,write,fd,buf,sizeof(buf));
  buffer_puts(&b,path.s);
  buffer_puts(&b,"/");
  buffer_puts(&b,name.s);
  buffer_puts(&b,"\n");
  mywrite(fd,buf,sizeof(buf));
  close(fd);

  log(B("Successfully registered ",name.s,"."));
  return 0;
}
